@@ -1,3 +1,4 @@
+import enum
 import pccm
 from pccm.test_data import PTest1, Test2
 
@@ -11,7 +12,7 @@ class Test3(pccm.Class):
                         "int",
                         pyanno="int",
                         mw_metas=[pccm.pybind.Pybind11PropMeta()])
-        self.add_member("private_prop_", "int", "0")
+        self.add_member("private_prop_", "int", "0", doc="a private prop. \ndetails.")
 
     @pccm.pybind.mark
     @pccm.member_function(inline=True)
@@ -41,7 +42,6 @@ class Test3(pccm.Class):
         code.arg("val", "int")
         return code 
 
-
 class Test4(Test3):
     def __init__(self):
         super().__init__()
@@ -65,10 +65,22 @@ class Test4(Test3):
         """).arg("a,b", "int").ret("int")
         return code
 
+    @pccm.pybind.mark(nogil=True)
+    @pccm.member_function(inline=True, name="add2", const=True)
+    def add2_overload(self):
+        code = pccm.FunctionCode("")
+        code.raw("""
+        return a + b;
+        """).arg("a,b", "int").ret("int")
+        return code
+
+
 class PbTestVirtual(pccm.Class, pccm.pybind.PybindClassMixin):
     def __init__(self):
         super().__init__()
         self.add_pybind_member("a", "int", "0")
+        self.add_enum_class("EnumClassExample", [("kValue1", 1), ("kValue2", 2)])
+        self.add_enum("EnumExample", [("kValue1", 1), ("kValue2", 2)])
 
     @pccm.pybind.mark(virtual=True)
     @pccm.member_function(virtual=True)
